@@ -43,6 +43,7 @@ from pwman.ui import tools
 from pwman.ui.tools import CliMenu
 from pwman.ui.tools import CliMenuItem
 from pwman.ui.tools import CLICallback
+from colorama import Fore
 
 try:
     import readline
@@ -110,7 +111,7 @@ class PwmanCli(cmd.Cmd):
         """
         if argsgiven == 1:
             length = tools.getinput("Password length (default 7): ", "7")
-            length = int(length)
+            length = len(length)
             password, dumpme = generator.generate_password(length, length,
                                                            True, leetify,
                                                            numerics,
@@ -122,13 +123,15 @@ class PwmanCli(cmd.Cmd):
                                      tools._defaultwidth, False)
         if len(password) == 0:
             length = tools.getinput("Password length (default 7): ", "7")
-            length = int(length)
-
-            (password, dumpme) = generator.generate_password(length, length,
-                                                             True, leetify,
-                                                             numerics,
-                                                             special_signs)
-            print "New password: %s" % (password)
+            if length:
+                length = int(length)
+            else:
+                length = 7
+            password, dumpme = generator.generate_password(length, length,
+                                                           True, leetify,
+                                                           numerics,
+                                                           special_signs)
+        print "New password: %s" % (password)
         return password
 
     def get_url(self, default=""):
@@ -171,16 +174,16 @@ class PwmanCli(cmd.Cmd):
 
     def print_node(self, node):
         width = str(tools._defaultwidth)
-        print "Node %d." % (node.get_id())
-        print ("%"+width+"s %s") % (tools.typeset("Username:", tools.ANSI.Red),
+        print "Node %d." % (node._id)
+        print ("%"+width+"s %s") % (tools.typeset("Username:", Fore.RED),
                                     node.get_username())
-        print ("%"+width+"s %s") % (tools.typeset("Password:", tools.ANSI.Red),
+        print ("%"+width+"s %s") % (tools.typeset("Password:", Fore.RED),
                                     node.get_password())
-        print ("%"+width+"s %s") % (tools.typeset("Url:", tools.ANSI.Red),
+        print ("%"+width+"s %s") % (tools.typeset("Url:", Fore.RED),
                                     node.get_url())
-        print ("%"+width+"s %s") % (tools.typeset("Notes:", tools.ANSI.Red),
+        print ("%"+width+"s %s") % (tools.typeset("Notes:", Fore.RED),
                                     node.get_notes())
-        print tools.typeset("Tags: ", tools.ANSI.Red),
+        print tools.typeset("Tags: ", Fore.RED),
         for t in node.get_tags():
             print " %s \n" % t.get_name(),
 
@@ -453,7 +456,7 @@ class PwmanCli(cmd.Cmd):
 
                 fmt = "%%5d. %%-%ds %%-%ds" % (name_len, tagstring_len)
                 print tools.typeset(fmt % (n.get_id(), name, tagstring),
-                                    tools.ANSI.Yellow, False)
+                                    Fore.YELLOW, False)
                 i += 1
                 if i > rows-2:
                     i = 0
@@ -709,8 +712,13 @@ pwman> n {'leetify':False, 'numerics':True}"""
         initialize CLI interface, set up the DB
         connecion, see if we have xsel ...
         """
-        _dbwarning = "\n*** WARNNING: You are using the old db format which" \
-                     + " uses cPickle, please upgrade your db !!! ***"
+        _dbwarning = "\n*** WARNNING: You are using the old database format" \
+            + " which is unsecure." \
+            + " It's highly recommended to switch to the new database " \
+            + "format." \
+            + " Check the help (pwman3 -h) or look at the manpage which" \
+            + " explains how to proceed. ***"
+
         cmd.Cmd.__init__(self)
         self.intro = "%s %s (c) visit: %s %s" % (pwman.appname, pwman.version,
                                                  pwman.website, _dbwarning)
@@ -750,7 +758,8 @@ class PwmanCliNew(PwmanCli):
         self.hasxsel = hasxsel
         try:
             enc = CryptoEngine.get()
-            enc.set_callback(CLICallback())
+            #enc.set_callback(CLICallback())
+            enc._callback = CLICallback()
             self._db = db
             self._db.open()
         except Exception, e:
@@ -759,24 +768,91 @@ class PwmanCliNew(PwmanCli):
 
         try:
             readline.read_history_file(self._historyfile)
-        except NameError, e:
+        except IOError, e:
             pass
 
         self.prompt = "pwman> "
 
+    def do_copy(self, args):
+        if self.hasxsel:
+            ids = self.get_ids(args)
+            if len(ids) > 1:
+                print "Can copy only 1 password at a time..."
+                return None
+            try:
+                node = self._db.getnodes(ids)
+                tools.text_to_clipboards(node[0].password)
+                print "copied password for {}@{} clipboard".format(
+                    node[0].username, node[0].url)
+
+                print "erasing in 10 sec..."
+                time.sleep(10)
+                tools.text_to_clipboards("")
+            except Exception, e:
+                self.error(e)
+        else:
+            print "Can't copy to clipboard, no xsel found in the system!"
+
+    def do_open(self, args):
+        ids = self.get_ids(args)
+        if not args:
+            self.help_open()
+            return
+        if len(ids) > 1:
+            print "Can open only 1 link at a time ..."
+            return None
+        try:
+            node = self._db.getnodes(ids)
+            url = node[0].url
+            tools.open_url(url)
+        except Exception, e:
+            self.error(e)
+
+    def do_edit(self, arg):
+        ids = self.get_ids(arg)
+        for i in ids:
+            try:
+                i = int(i)
+                node = self._db.getnodes([i])[0]
+                menu = CliMenu()
+                print "Editing node %d." % (i)
+
+                menu.add(CliMenuItem("Username", self.get_username,
+                                     node.username,
+                                     node.username))
+                menu.add(CliMenuItem("Password", self.get_password,
+                                     node.password,
+                                     node.password))
+                menu.add(CliMenuItem("Url", self.get_url,
+                                     node.url,
+                                     node.url))
+                menunotes = CliMenuItem("Notes", self.get_notes,
+                                        node.notes,
+                                        node.notes)
+                menu.add(menunotes)
+                menu.add(CliMenuItem("Tags", self.get_tags,
+                                     node.tags,
+                                     node.tags))
+                menu.runner(node)
+                self._db.editnode(i, node)
+                # when done with node erase it
+                zerome(node._password)
+            except Exception, e:
+                self.error(e)
+
     def print_node(self, node):
         width = str(tools._defaultwidth)
-        print "Node %d." % (node.get_id())
-        print ("%"+width+"s %s") % (tools.typeset("Username:", tools.ANSI.Red),
-                                    node.get_username())
-        print ("%"+width+"s %s") % (tools.typeset("Password:", tools.ANSI.Red),
-                                    node.get_password())
-        print ("%"+width+"s %s") % (tools.typeset("Url:", tools.ANSI.Red),
-                                    node.get_url())
-        print ("%"+width+"s %s") % (tools.typeset("Notes:", tools.ANSI.Red),
-                                    node.get_notes())
-        print tools.typeset("Tags: ", tools.ANSI.Red),
-        for t in node.get_tags():
+        print "Node %d." % (node._id)
+        print ("%"+width+"s %s") % (tools.typeset("Username:", Fore.RED),
+                                    node.username)
+        print ("%"+width+"s %s") % (tools.typeset("Password:", Fore.RED),
+                                    node.password)
+        print ("%"+width+"s %s") % (tools.typeset("Url:", Fore.RED),
+                                    node.url)
+        print ("%"+width+"s %s") % (tools.typeset("Notes:", Fore.RED),
+                                    node.notes)
+        print tools.typeset("Tags: ", Fore.RED),
+        for t in node.tags:
             print " %s " % t
         print
 
@@ -818,7 +894,6 @@ class PwmanCliNew(PwmanCli):
 
     def get_tags(self, default=None):
         defaultstr = ''
-
         if default:
             for t in default:
                 defaultstr += "%s " % (t)
@@ -844,10 +919,8 @@ class PwmanCliNew(PwmanCli):
 
         taglist = tools.getinput("Tags: ", defaultstr, complete)
         tagstrings = taglist.split()
-        tags = []
-        for tn in tagstrings:
-            _Tag = Tag(tn)
-            tags.append(_Tag)
+        tags = [Tag(tn) for tn in tagstrings]
+
         return tags
 
     def do_list(self, args):
@@ -861,10 +934,11 @@ class PwmanCliNew(PwmanCli):
                 rows, cols = 18, 80  # fix this !
             nodeids = self._db.listnodes()
             nodes = self._db.getnodes(nodeids)
+
             cols -= 8
             i = 0
             for n in nodes:
-                tags = n.get_tags()
+                tags = n.tags
                 tags = filter(None, tags)
                 tagstring = ''
                 first = True
@@ -875,7 +949,7 @@ class PwmanCliNew(PwmanCli):
                         first = False
                     tagstring += t
 
-                name = "%s@%s" % (n.get_username(), n.get_url())
+                name = "%s@%s" % (n.username, n.url)
 
                 name_len = cols * 2 / 3
                 tagstring_len = cols / 3
@@ -884,9 +958,9 @@ class PwmanCliNew(PwmanCli):
                 if len(tagstring) > tagstring_len:
                     tagstring = tagstring[:tagstring_len-3] + "..."
                 fmt = "%%5d. %%-%ds %%-%ds" % (name_len, tagstring_len)
-                formatted_entry = tools.typeset(fmt % (n.get_id(),
+                formatted_entry = tools.typeset(fmt % (n._id,
                                                 name, tagstring),
-                                                tools.ANSI.Yellow, False)
+                                                Fore.YELLOW, False)
                 print formatted_entry
                 i += 1
                 if i > rows-2:
@@ -911,7 +985,7 @@ class PwmanCliNew(PwmanCli):
             if len(tags) == 0:
                 print "None",
             for t in tags:
-                print "%s " % (t.get_name()),
+                print "%s " % (t.name),
             print
         except Exception, e:
             self.error(e)
@@ -951,10 +1025,9 @@ class PwmanCliNew(PwmanCli):
             url = self.get_url()
             notes = self.get_notes()
             node = NewNode(username, password, url, notes)
-            tags = self.get_tags()
-            node.set_tags(tags)
+            node.tags = self.get_tags()
             self._db.addnodes([node])
-            print "Password ID: %d" % (node.get_id())
+            print "Password ID: %d" % (node._id)
             # when done with node erase it
             zerome(password)
         except Exception, e:
@@ -969,3 +1042,16 @@ class PwmanCliNew(PwmanCli):
                 zerome(node[0]._password)
             except Exception, e:
                 self.error(e)
+
+    def do_delete(self, arg):
+        ids = self.get_ids(arg)
+        try:
+            nodes = self._db.getnodes(ids)
+            for n in nodes:
+                b = tools.getyesno("Are you sure you want to delete '%s@%s'?"
+                                   % (n.username, n.url), False)
+                if b is True:
+                    self._db.removenodes([n])
+                    print "%s@%s deleted" % (n.username, n.url)
+        except Exception, e:
+            self.error(e)
